#!/usr/bin/env groovy

final def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
final def ahptool = isWindows ? 'ahptool.cmd' : 'ahptool'

//------------------------------------------------------------------------------
// Utility Classes
//------------------------------------------------------------------------------

class PMDHandler extends org.xml.sax.helpers.DefaultHandler {

    def findingFile
    def findingLine
    def findingName
    def findingSeverity
    def addFinding
    def type

    PMDHandler(def addFinding, def type) {
        this.addFinding = addFinding
        this.type = type
    }

    void startElement(String ns, String localName, String qName, org.xml.sax.Attributes attribs) {
        switch (qName) {
            case 'file':
                findingFile = attribs.getValue('name')
                break
            case 'violation':
                findingLine = attribs.getValue('beginline')
                findingName = attribs.getValue('rule')
                if (type == 'priority') {
                    findingSeverity = 'Priority ' + attribs.getValue('priority')
                }
                else {
                    findingSeverity = attribs.getValue('ruleset')
                }
                break
        }
    }

    void endElement(String ns, String localName, String qName) {
        switch (qName) {
            case 'violation':
                addFinding('', findingFile, findingLine, findingName, findingSeverity, '', '')
                break
        }
    }
}

//------------------------------------------------------------------------------
// Utility methods
//------------------------------------------------------------------------------

final def getStepProperties = {
    def props = [:]

    def ahptoolProc = [ahptool, "getStepProperties"].execute()
    ahptoolProc.consumeProcessErrorStream(System.out);
    def propertiesXml = ahptoolProc.text
    ahptoolProc.waitFor()
    if (ahptoolProc.exitValue()) {
        throw new Exception("Failed to get step properties from ahptool: " + propertiesXml)
    }
    new XmlSlurper().parseText(propertiesXml).property.each{ propElem ->
        props[propElem.@name.text()] = propElem.text()
    }

    return props;
}

final def workDir = new File('.').canonicalFile
final def workDirPath = workDir.getCanonicalPath()
final def out = System.out

final File resourceHome = new File(System.getenv()['PLUGIN_HOME'])
final def props = getStepProperties()

final def buildLifeId = System.getenv()['AH_BUILD_LIFE_ID']
final def outputFile = new File(workDir, props['outputFile'])
final def reportName = props['reportName'] ? props['reportName'] : 'PMD Report'
final def severityType = props['severityType']

if (!outputFile.isFile()) {
    if (!outputFile.exists()) {
        println("PMD report file $outputFile.absolutePath does not exist!")
    }
    else {
        println("PMD report file $outputFile.absolutePath is not a file!")
    }
    System.exit(1)
}

// define a closure for parsing the PMD output file and writing out Anthill XML format
def xml = new groovy.xml.StreamingMarkupBuilder().bind {
    mkp.xmlDeclaration()

    analytics (
        name:reportName, buildLifeId:buildLifeId, type:'PMD') {

        def addFinding = { findingId, findingFile, findingLine, findingName, findingSeverity, findingDesc, findingStatus ->
            if (findingFile.startsWith(workDirPath)) {
                findingFile = findingFile.substring(workDirPath.length() + 1)
            }
            finding() {
                id(findingId)
                file(findingFile)
                line(findingLine)
                name(findingName)
                severity(findingSeverity)
                description(findingDesc)
                status(findingStatus)
            }
        }

        def handler = new PMDHandler(addFinding, severityType)
        def reader = javax.xml.parsers.SAXParserFactory.newInstance().newSAXParser().XMLReader
        reader.setContentHandler(handler)
        def inputStream = new FileInputStream(outputFile)
        reader.parse(new org.xml.sax.InputSource(inputStream))
    }
}

//------------------------------------------------------------------------------
// Construct XML message and upload to AHP
//------------------------------------------------------------------------------

if (xml) {
    //println(xml)
    println("Uploading PMD results to server as report '" + reportName + "' ....")

    pb = new ProcessBuilder([ahptool, '-report', reportName, 'addSourceAnalytics', '-']).directory(workDir)
    proc = pb.start()
    proc.consumeProcessOutput(out, out);
    proc.withWriter{ it << xml }
    proc.waitFor()
    if (proc.exitValue()) {
        println("PMD results upload to server failed with exit code: " + proc.exitValue())
        System.exit(proc.exitValue())
    }
    else {
        println("PMD results uploaded successfully")
    }
}
else {
    println "No PMD results to upload"
}
System.exit(proc.exitValue())
